@@ -2,9 +2,9 @@
 ## package 'secr'
 ## D.design.MS.R
 ## Prepare density design matrix
-## 
+##
 ## force levels=sessionlevels for dframe$session 2010 02 25
-
+## temporary raise error if mask covariates and multiple sessions
 ############################################################################################
 ## NOTE does not standardize sessioncov, maskcov
 ############################################################################################
@@ -17,8 +17,9 @@ D.designdata <- function (mask, Dmodel, grps, sessionlevels, sessioncov = NULL)
     stdfn <- function(x) if (is.numeric(x)) (x-mean(x))/sqrt(var(x)) else x
 
     #######################################
-    findvars <- function (cov, vars, dimcov, std=TRUE) 
+    findvars <- function (cov, vars, dimcov, std=TRUE)
     ## function to add covariates to a design data frame
+    ## DOES NOT HANDLE MULTIPLE SESSIONS, EACH WITH OWN MASK 2010-04-25
     {
         found <- names(cov) %in% vars
         if (is.data.frame(cov) & any(found)) {
@@ -26,7 +27,7 @@ D.designdata <- function (mask, Dmodel, grps, sessionlevels, sessioncov = NULL)
             values <- as.data.frame(cov[,found])
             names(values) <- found
             if (length(values)>0) {
-                for (i in 1:ncol(values)) {    
+                for (i in 1:ncol(values)) {
                     vals <- values[,i]
                     if (!is.factor(vals) & std) vals <- stdfn(vals)
                     eval(parse(text=paste ('dframe$', found[i],
@@ -37,7 +38,7 @@ D.designdata <- function (mask, Dmodel, grps, sessionlevels, sessioncov = NULL)
         }
     }
     #######################################
-    
+
     MS <- length(sessionlevels) > 1  ## !inherits(mask, 'mask')
     if (MS) nmask <- max(sapply(mask, nrow))
     else nmask <- nrow(mask)
@@ -70,25 +71,26 @@ D.designdata <- function (mask, Dmodel, grps, sessionlevels, sessioncov = NULL)
       y = rep(as.vector(unlist(y)), ngrp)
     ))
 
-    if ('g' %in% vars) {   
+    if ('g' %in% vars) {
         if (length(grps)<1) stop('No groups specified')
         dframe$g <- insertdim(factor(grps), 2, dims)
-    } 
+    }
 
     ## force levels=sessionlevels 2010 02 25
     if ('session' %in% vars) {
        dframe$session <- insertdim(factor(sessionlevels, levels=sessionlevels), 3, dims)
-    } 
+    }
 
-    ## all autovars should have now been dealt with 
+    ## all autovars should have now been dealt with
     vars <- vars[!vars %in% c('g','x','y','session')]
 
     if (!is.null(sessioncov)) {
-       findvars (sessioncov, vars, 3, std = FALSE)   
+        findvars (sessioncov, vars, 3, std = FALSE)
     }
 
-    if (!is.null(covariates(mask))) {
-       findvars (covariates(mask), vars, 1, std = FALSE)   
+    if ((!is.null(covariates(mask))) & (length(vars>0))) {
+        if (MS) stop ('D.designdata does not yet handle mask covariates for  multiple sessions')
+        findvars (covariates(mask), vars, 1, std = FALSE)
     }
 
     if (length(vars)>0) stop (paste(paste(vars,collapse=','),'not found'))
