@@ -1,8 +1,9 @@
 ############################################################################################
 ## package 'secr'
 ## secr.make.newdata.R
-## last changed 2009 01 27, 2009 04 07, 2009 06 11, 2009 07 23, 2009 08 16, 2009 09 04, 
-## 2009 09 10
+## last changed 
+## 2009 12 13 (mixtures)
+## 2010 03 10 'T'
 ## Create (neutral) design data suitable for 'predict' 
 ############################################################################################
 
@@ -14,6 +15,8 @@ secr.make.newdata <- function (object) {
     groups <- object$groups
     timecov <- object$timecov
     sessioncov <- object$sessioncov
+    nmix <- object$details$nmix
+    if(is.null(nmix)) nmix <- 1
 
     findvars.MS <- function (cov, vars, dimcov, use.all) {
         ## function to add covariates to a design data frame 'dframe'
@@ -58,27 +61,26 @@ secr.make.newdata <- function (object) {
             vars <<- vars[!(vars %in% found)]
         }
     }
-
     nocc <- max(n.occasion (capthist))
     grps <- group.levels(capthist, groups)
     ngrp <- max(1, length(grps))
     sessions <- session(capthist)
     R <- length(sessions)
-    dims <- c(R, ngrp)
-
-    if (ngrp <= 1) {
-        newdata <- expand.grid(session=sessions)
+    dims <- c(R, ngrp, nmix)
+    basevars <- list(session=sessions)
+    if (ngrp>1) basevars$g <- factor(grps)
+    if (nmix==2) basevars$h2 <- factor(1:2)
+    if (nmix==3) basevars$h3 <- factor(1:3)
+    newdata <- expand.grid(basevars)
+    nr <- nrow(newdata)  ## one row for each session, group and mixture
+    if (ngrp==1)
         findvars.MS (covariates(capthist), vars, 1, FALSE) ## check for indiv cov  
-    }
-    else 
-        newdata <- expand.grid(session=sessions, g=factor(grps))
-
-    nr <- nrow(newdata)  ## one row for each session and group
     
     for (v in vars) {
         if (v=='x') newdata$x <- rep(0,nr)   # mean attr(mask,'meanSD')[1,'x']
         if (v=='y') newdata$y <- rep(0,nr)   # mean attr(mask,'meanSD')[1,'y']
         if (v=='t') newdata$t <- rep(factor(1, levels=1:nocc), nr)   ## mod 2009 09 03
+        if (v=='T') newdata$T <- rep(0:(nocc-1), nr)   ## 2010 03 10
         if (v=='b') newdata$b <- rep(factor(0, levels=c(0,1)),nr)    # naive
         if (v=='B') newdata$B <- rep(factor(0, levels=c(0,1)),nr)    # naive
 #        if (v=='bk') newdata$bk <- rep(factor(0, levels=c(0,1)),nr)   # naive
@@ -88,8 +90,7 @@ secr.make.newdata <- function (object) {
     }
 
     ## all autovars should now have been dealt with 
-#    vars <- vars[!vars %in% c('g','x','y','session','t','b','bk','B','Bk','tcov','kcov')]
-    vars <- vars[!vars %in% c('g','x','y','session','t','b','B','tcov','kcov')]
+    vars <- vars[!vars %in% c('g','x','y','session','t','T','b','B','tcov','kcov','h2','h3')]
 
     findvars.MS (sessioncov, vars, 1, TRUE)   
     findvars.MS (timecov, vars, 1, FALSE)   
@@ -97,7 +98,6 @@ secr.make.newdata <- function (object) {
 
     ## default all remaining vars to numeric zero
     for (v in vars) newdata[,v] <- rep(0,nr)
-
     newdata
 }
 ############################################################################################
